package de.intevation.flys.artifacts.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import de.intevation.artifacts.Artifact;

import de.intevation.flys.artifacts.access.FlowVelocityAccess;

import de.intevation.flys.model.DischargeZone;
import de.intevation.flys.model.FlowVelocityModel;
import de.intevation.flys.model.FlowVelocityModelValue;
import de.intevation.flys.model.River;

import org.apache.log4j.Logger;


public class FlowVelocityCalculation extends Calculation {

    private static final Logger logger =
        Logger.getLogger(FlowVelocityCalculation.class);


    public CalculationResult calculate(FlowVelocityAccess access) {
        logger.info("FlowVelocityCalculation.calculate");

        int[] mainIds  = access.getMainChannels();
        int[] totalIds = access.getTotalChannels();

        if (logger.isDebugEnabled()) {
            Artifact a = access.getArtifact();
            logger.debug("Artifact '" + a.identifier() + "' contains:");
            if (mainIds != null) {
                logger.debug("   " + mainIds.length + " main channel ids");
            }

            if (totalIds != null) {
                logger.debug("   " + totalIds.length + " total channel ids");
            }
        }

        List<DischargeZone>     zones  = getDischargeZones(mainIds, totalIds);
        List<FlowVelocityModel> models = getFlowVelocityModels(access, zones);

        return buildCalculationResult(access, models);
    }


    protected List<DischargeZone> getDischargeZones(
        int[] mainIds,
        int[] totalIds
    ) {
        List<DischargeZone> zones = new ArrayList<DischargeZone>();

        if (mainIds != null) {
            for (int id: mainIds) {
                DischargeZone zone = DischargeZone.getDischargeZoneById(id);

                if (zone != null) {
                    zones.add(zone);
                }
            }
        }

        if (totalIds != null) {
            for (int id: totalIds) {
                DischargeZone zone = DischargeZone.getDischargeZoneById(id);

                if (zone != null) {
                    zones.add(zone);
                }
            }
        }

        return zones;
    }


    protected List<FlowVelocityModel> getFlowVelocityModels(
        FlowVelocityAccess  access,
        List<DischargeZone> zones
    ) {
        String riverName = access.getRiver();
        if (riverName == null) {
            logger.warn("No river name found");
            return Collections.<FlowVelocityModel>emptyList();
        }

        River river = RiverFactory.getRiver(riverName);
        if (river == null) {
            logger.warn("No such river: " + riverName);
            return Collections.<FlowVelocityModel>emptyList();
        }

        List<FlowVelocityModel> models = new ArrayList<FlowVelocityModel>();

        for (DischargeZone zone: zones) {
            List<FlowVelocityModel> model =
                FlowVelocityModel.getModels(river, zone);

            if (model != null) {
                models.addAll(model);
            }
        }

        return models;
    }


    protected void prepareData(
        FlowVelocityData  data,
        FlowVelocityModel model,
        double kmLo,
        double kmHi
    ) {
        List<FlowVelocityModelValue> values =
            FlowVelocityModelValue.getValues(model, kmLo, kmHi);

        logger.debug("Found " + values.size() + " values for model.");

        for (FlowVelocityModelValue value: values) {
            data.addKM(value.getStation().doubleValue());
            data.addQ(value.getQ().doubleValue());
            data.addVTotal(value.getTotalChannel().doubleValue());
            data.addVMain(value.getMainChannel().doubleValue());
            data.addTauMain(value.getShearStress().doubleValue());
        }

        DischargeZone zone = model.getDischargeZone();
        String        lo   = zone.getLowerDischarge();
        String        hi   = zone.getUpperDischarge();

        if (lo.equals(hi)) {
            data.setZone(lo);
        }
        else {
            data.setZone(lo + " - " + hi);
        }
    }


    protected CalculationResult buildCalculationResult(
        FlowVelocityAccess         access,
        List<FlowVelocityModel> models
    ) {
        double kmLo = access.getLowerKM();
        double kmHi = access.getUpperKM();

        logger.debug("Prepare data for km range: " + kmLo + " - " + kmHi);

        FlowVelocityData[] data = new FlowVelocityData[models.size()];

        for (int i = 0, n = models.size(); i < n; i++) {
            FlowVelocityData d = new FlowVelocityData();

            prepareData(d, models.get(i), kmLo, kmHi);

            data[i] = d;
        }

        logger.debug("Calculation contains " + data.length + " data items.");

        return new CalculationResult(data, this);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
